##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""CookieCrumbler tests.
"""

import unittest

from six import StringIO
from six.moves.urllib.parse import quote

from zope.component import eventtesting
from zope.interface.verify import verifyClass
from zope.testing.cleanup import cleanUp

from ..utils import base64_encode


def makerequest(root, stdout, stdin=None):
    # Customized version of Testing.makerequest.makerequest()
    from ZPublisher.HTTPRequest import HTTPRequest
    from ZPublisher.HTTPResponse import HTTPResponse

    resp = HTTPResponse(stdout=stdout)
    environ = {}
    environ['SERVER_NAME'] = 'example.com'
    environ['SERVER_PORT'] = '80'
    environ['REQUEST_METHOD'] = 'GET'
    if stdin is None:
        stdin = StringIO('')  # Empty input
    req = HTTPRequest(stdin, environ, resp)
    req['PARENTS'] = [root]
    return req


class CookieCrumblerTests(unittest.TestCase):

    def _getTargetClass(self):
        from ..CookieCrumbler import CookieCrumbler
        return CookieCrumbler

    def _makeOne(self, *args, **kw):
        return self._getTargetClass()(*args, **kw)

    def setUp(self):
        from zope.component import provideHandler
        from zope.interface.interfaces import IObjectEvent

        from ..CookieCrumbler import handleCookieCrumblerEvent
        from ..interfaces import ICookieCrumbler

        self._finally = None

        eventtesting.setUp()
        provideHandler(handleCookieCrumblerEvent,
                       adapts=(ICookieCrumbler, IObjectEvent))

    def tearDown(self):
        from AccessControl.SecurityManagement import noSecurityManager

        if self._finally is not None:
            self._finally()

        noSecurityManager()
        cleanUp()

    def _makeSite(self):
        from OFS.DTMLMethod import DTMLMethod
        from OFS.Folder import Folder
        from OFS.userfolder import UserFolder

        class TestFolder(Folder):
            def getPhysicalPath(self):
                return ()

        root = TestFolder()
        root.isTopLevelPrincipiaApplicationObject = 1  # User folder needs this
        root._View_Permission = ('Anonymous',)

        users = UserFolder()
        users._setId('acl_users')
        users._doAddUser('abraham', 'pass-w', ('Patriarch',), ())
        users._doAddUser('isaac', 'pass-w', ('Son',), ())
        root._setObject(users.id, users)

        cc = self._makeOne()
        root._setObject(cc.id, cc)

        index = DTMLMethod()
        index.munge('This is the default view')
        index._setId('index_html')
        root._setObject(index.getId(), index)

        login = DTMLMethod()
        login.munge('Please log in first.')
        login._setId('login_form')
        root._setObject(login.getId(), login)

        protected = DTMLMethod()
        protected._View_Permission = ('Manager',)
        protected.munge('This is the protected view')
        protected._setId('protected')
        root._setObject(protected.getId(), protected)

        req = makerequest(root, StringIO())
        self._finally = req.close

        credentials = quote(base64_encode(b'abraham:pass-w'))

        return root, cc, req, credentials

    def test_interfaces(self):
        from ..interfaces import ICookieCrumbler

        verifyClass(ICookieCrumbler, self._getTargetClass())

    def test_defaults(self):
        klass = self._getTargetClass()
        cc = self._makeOne()
        self.assertEqual(cc.getId(), klass.id)
        self.assertEqual(cc.title, '')

    def test_factory(self):
        from ..CookieCrumbler import manage_addCC
        _root, _cc, req, _credentials = self._makeSite()
        manage_addCC(_root, 'new_cc', title='I am a Cookie Crumbler')

        self.assertIn('new_cc', _root.objectIds())
        self.assertEqual(_root.new_cc.getId(), 'new_cc')
        self.assertEqual(_root.new_cc.title, 'I am a Cookie Crumbler')

    def testNoCookies(self):
        # verify the cookie crumbler doesn't break when no cookies are given
        _root, _cc, req, _credentials = self._makeSite()
        req.traverse('/')
        self.assertEqual(req['AUTHENTICATED_USER'].getUserName(),
                         'Anonymous User')

    def testCookieLogin(self):
        # verify the user and auth cookie get set
        _root, _cc, req, credentials = self._makeSite()

        req.cookies['__ac_name'] = 'abraham'
        req.cookies['__ac_password'] = 'pass-w'
        req.traverse('/')

        self.assertTrue('AUTHENTICATED_USER' in req)
        self.assertEqual(req['AUTHENTICATED_USER'].getUserName(), 'abraham')
        resp = req.response
        self.assertTrue('__ac' in resp.cookies)
        self.assertEqual(resp.cookies['__ac']['value'], credentials)
        self.assertEqual(resp.cookies['__ac'][
            normalizeCookieParameterName('path')], '/')

    def testCookieResume(self):
        # verify the cookie crumbler continues the session
        _root, _cc, req, credentials = self._makeSite()
        req.cookies['__ac'] = credentials
        req.traverse('/')
        self.assertTrue('AUTHENTICATED_USER' in req)
        self.assertEqual(req['AUTHENTICATED_USER'].getUserName(), 'abraham')

    def testPasswordShredding(self):
        # verify the password is shredded before the app gets the request
        _root, _cc, req, _credentials = self._makeSite()
        req.cookies['__ac_name'] = 'abraham'
        req.cookies['__ac_password'] = 'pass-w'
        self.assertTrue('__ac_password' in req)
        req.traverse('/')
        self.assertFalse('__ac_password' in req)
        self.assertFalse('__ac' in req)

    def testCredentialsNotRevealed(self):
        # verify the credentials are shredded before the app gets the request
        _root, _cc, req, credentials = self._makeSite()
        req.cookies['__ac'] = credentials
        self.assertTrue('__ac' in req)
        req.traverse('/')
        self.assertFalse('__ac' in req)

    def testCacheHeaderAnonymous(self):
        # Should not set cache-control
        _root, _cc, req, _credentials = self._makeSite()
        req.traverse('/')
        self.assertEqual(req.response.headers.get('cache-control', ''), '')

    def testCacheHeaderLoggingIn(self):
        # Should set cache-control
        _root, _cc, req, _credentials = self._makeSite()
        req.cookies['__ac_name'] = 'abraham'
        req.cookies['__ac_password'] = 'pass-w'
        req.traverse('/')
        self.assertEqual(req.response.headers.get('cache-control', ''),
                         'private')

    def testCacheHeaderAuthenticated(self):
        # Should set cache-control
        _root, _cc, req, credentials = self._makeSite()
        req.cookies['__ac'] = credentials
        req.traverse('/')
        self.assertEqual(req.response.headers.get('cache-control', ''),
                         'private')

    def testCacheHeaderDisabled(self):
        # Should not set cache-control
        _root, cc, req, credentials = self._makeSite()
        cc.cache_header_value = ''
        req.cookies['__ac'] = credentials
        req.traverse('/')
        self.assertEqual(req.response.headers.get('cache-control', ''), '')

    def testLoginRatherThanResume(self):
        # When the user presents both a session resume and new
        # credentials, choose the new credentials (so that it's
        # possible to log in without logging out)
        _root, _cc, req, credentials = self._makeSite()
        req.cookies['__ac_name'] = 'isaac'
        req.cookies['__ac_password'] = 'pass-w'
        req.cookies['__ac'] = credentials
        req.traverse('/')

        self.assertTrue('AUTHENTICATED_USER' in req)
        self.assertEqual(req['AUTHENTICATED_USER'].getUserName(), 'isaac')

    def test_before_traverse_hooks(self):
        from OFS.Folder import Folder

        container = Folder()
        cc = self._makeOne()

        marker = []
        bt_before = getattr(container, '__before_traverse__', marker)
        self.assertTrue(bt_before is marker)

        container._setObject(cc.id, cc)

        bt_added = getattr(container, '__before_traverse__')
        self.assertEqual(len(bt_added.items()), 1)
        k, v = list(bt_added.items())[0]
        self.assertTrue(k[1].startswith(self._getTargetClass().meta_type))
        self.assertEqual(v.name, cc.id)

        container._delObject(cc.id)

        bt_removed = getattr(container, '__before_traverse__')
        self.assertEqual(len(bt_removed.items()), 0)


# compatibility
try:
    # Zope 4.8+/5.5+
    from ZPublisher.cookie import normalizeCookieParameterName
except ImportError:
    def normalizeCookieParameterName(name):
        return name


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(CookieCrumblerTests),
        ))



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Portal services base objects
"""

try:
    import Products.ZSQLMethods  # noqa

    from . import FSZSQLMethod
    HAVE_ZSQL = True
except ImportError:
    HAVE_ZSQL = False

# Make sure security is initialized
from . import ActionInformation
from . import ActionsTool
from . import CachingPolicyManager
from . import CatalogTool
from . import CMFBTreeFolder
from . import ContentTypeRegistry
from . import CookieCrumbler
from . import DirectoryView
from . import DiscussionTool
from . import FSDTMLMethod
from . import FSFile
from . import FSImage
from . import FSPageTemplate
from . import FSPropertiesObject
from . import FSPythonScript
from . import FSReSTMethod  # noqa
from . import FSSTXMethod  # noqa
from . import MemberDataTool
from . import MembershipTool
from . import PortalContent  # noqa
from . import PortalFolder
from . import PortalObject  # noqa
from . import RegistrationTool
from . import SkinsTool
from . import TypesTool
from . import UndoTool
from . import URLTool
from . import WorkflowTool
from . import utils
from .permissions import AddPortalFolders


tools = (MembershipTool.MembershipTool,
         RegistrationTool.RegistrationTool,
         WorkflowTool.WorkflowTool,
         CatalogTool.CatalogTool,
         DiscussionTool.DiscussionTool,
         ActionsTool.ActionsTool,
         UndoTool.UndoTool,
         SkinsTool.SkinsTool,
         MemberDataTool.MemberDataTool,
         TypesTool.TypesTool,
         URLTool.URLTool)

# BBB: oldstyle constructors
_EXTRA_CONSTRUCTORS = (PortalFolder.manage_addPortalFolder,
                       CMFBTreeFolder.manage_addCMFBTreeFolder)

# Because persistent objects may be out there which were
# created when the module was in that product, we need
# __module_aliases__ .
__module_aliases__ = (('Products.BTreeFolder2.CMFBTreeFolder',
                       'Products.CMFCore.CMFBTreeFolder'),)


def initialize(context):
    context.registerClass(
        DirectoryView.DirectoryView,
        constructors=(('manage_addDirectoryViewForm',
                       DirectoryView.manage_addDirectoryViewForm),
                      DirectoryView.manage_addDirectoryView,
                      DirectoryView.manage_listAvailableDirectories),
        icon='images/dirview.gif')

    context.registerClass(
        CookieCrumbler.CookieCrumbler,
        constructors=(CookieCrumbler.manage_addCCForm,
                      CookieCrumbler.manage_addCC),
        icon='images/cookie.gif')

    context.registerClass(
        ContentTypeRegistry.ContentTypeRegistry,
        constructors=(ContentTypeRegistry.manage_addRegistry,),
        icon='images/registry.gif')

    context.registerClass(
        CachingPolicyManager.CachingPolicyManager,
        constructors=(CachingPolicyManager.manage_addCachingPolicyManager,),
        icon='images/registry.gif')

    utils.registerIcon(ActionInformation.ActionCategory,
                       'images/cmf_action_category.gif', globals())
    utils.registerIcon(ActionInformation.Action,
                       'images/cmf_action.gif', globals())
    utils.registerIcon(TypesTool.FactoryTypeInformation,
                       'images/typeinfo.gif', globals())
    utils.registerIcon(TypesTool.ScriptableTypeInformation,
                       'images/typeinfo.gif', globals())
    utils.registerIcon(FSDTMLMethod.FSDTMLMethod,
                       'images/fsdtml.gif', globals())
    utils.registerIcon(FSPythonScript.FSPythonScript,
                       'images/fspy.gif', globals())
    utils.registerIcon(FSImage.FSImage,
                       'images/fsimage.gif', globals())
    utils.registerIcon(FSFile.FSFile,
                       'images/fsfile.gif', globals())
    utils.registerIcon(FSPageTemplate.FSPageTemplate,
                       'images/fspt.gif', globals())
    utils.registerIcon(FSPropertiesObject.FSPropertiesObject,
                       'images/fsprops.gif', globals())
    if HAVE_ZSQL:
        utils.registerIcon(FSZSQLMethod.FSZSQLMethod,
                           'images/fssqlmethod.gif', globals())

    utils.ToolInit('CMF Core Tool', tools=tools,
                   icon='tool.gif').initialize(context)

    # BBB: register oldstyle constructors
    utils.ContentInit('CMF Core Content', content_types=(),
                      permission=AddPortalFolders,
                      extra_constructors=_EXTRA_CONSTRUCTORS,
                      visibility=None).initialize(context)



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit tests for FSFile module.
"""

import os
import unittest
import warnings

import six

from zope.component import getSiteManager
from zope.datetime import rfc1123_date
from zope.testing.cleanup import cleanUp

from ..interfaces import ICachingPolicyManager
from .base.dummy import FAKE_ETAG
from .base.dummy import DummyCachingManager
from .base.dummy import DummyCachingManagerWithPolicy
from .base.testcase import FSDVTest
from .base.testcase import TransactionalTest


class FSFileTests(TransactionalTest, FSDVTest):

    def setUp(self):
        TransactionalTest.setUp(self)
        FSDVTest.setUp(self)

    def tearDown(self):
        cleanUp()
        FSDVTest.tearDown(self)
        TransactionalTest.tearDown(self)

    def _makeOne(self, id, filename):
        from ..FSFile import FSFile
        from ..FSMetadata import FSMetadata

        full_path = os.path.join(self.skin_path_name, filename)
        metadata = FSMetadata(full_path)
        metadata.read()
        return FSFile(id, full_path, properties=metadata.getProperties())

    def _extractFile(self, filename):
        path = os.path.join(self.skin_path_name, filename)
        f = open(path, 'rb')
        try:
            data = f.read()
        finally:
            f.close()

        return path, data

    def test_ctor(self):
        _path, ref = self._extractFile('test_file.swf')

        file = self._makeOne('test_file', 'test_file.swf')
        file = file.__of__(self.app)

        self.assertEqual(file.get_size(), len(ref))
        self.assertEqual(file._readFile(0), ref)

    def test_bytes(self):
        _path, ref = self._extractFile('test_file.swf')
        file = self._makeOne('test_file', 'test_file.swf')
        file = file.__of__(self.app)
        self.assertEqual(len(bytes(file)), len(bytes(ref)))

    def test_str(self):
        from ZPublisher.HTTPRequest import default_encoding

        if six.PY2:
            # This test only makes sense under Python 2. The file
            # being loaded is a binary string, which is what ``str``
            # under Python 2 returns. Under Python 3 this will lead
            # to a decoding attempt, which will fail or destroy the data.
            _path, ref = self._extractFile('test_file.swf')
            file = self._makeOne('test_file', 'test_file.swf')
            file = file.__of__(self.app)
            self.assertEqual(len(str(file)), len(str(ref)))

        encoded = b'Th\xc3\xaes \xc3\xaes s\xc3\xb6me t\xc3\xa9xt.'

        # This file contains UTF-8 text data
        file = self._makeOne('test_text', 'test_text.txt')
        data = str(file).strip()

        self.assertIsInstance(data, str)
        if six.PY2:
            self.assertEqual(data, encoded)
        else:
            self.assertEqual(data, encoded.decode(default_encoding))

        # This file contains latin-1 test data
        file = self._makeOne('test_text', 'test_text_latin1.txt')
        data = str(file).strip()

        self.assertIsInstance(data, str)
        if six.PY2:
            l1_data = encoded.decode(default_encoding).encode('latin-1')
            self.assertEqual(data, l1_data)
        else:
            self.assertEqual(data, encoded.decode(default_encoding))

        # This file contains non-text binary data
        file = self._makeOne('test_binary', 'test_image.gif')
        _path, real_data = self._extractFile('test_image.gif')

        self.assertIsInstance(data, str)
        if six.PY2:
            self.assertEqual(str(file), real_data)
        else:
            # Calling ``__str__`` on non-text binary data in Python 3
            # will raise a DeprecationWarning because it makes no sense.
            # Ignore it here, the warning is irrelevant for the test.
            with warnings.catch_warnings():
                warnings.simplefilter('ignore')
                self.assertRaises(UnicodeDecodeError, file.__str__)

    def test_index_html(self):
        path, ref = self._extractFile('test_file.swf')
        mod_time = os.stat(path)[8]

        file = self._makeOne('test_file', 'test_file.swf')
        file = file.__of__(self.app)

        data = file.index_html(self.REQUEST, self.RESPONSE)

        self.assertEqual(len(data), len(ref))
        self.assertEqual(data, ref)
        # ICK!  'HTTPResponse.getHeader' doesn't case-flatten the key!
        self.assertEqual(self.RESPONSE.getHeader('Content-Length'.lower()),
                         str(len(ref)))
        self.assertEqual(self.RESPONSE.getHeader('Content-Type'.lower()),
                         'application/octet-stream')
        self.assertEqual(self.RESPONSE.getHeader('Last-Modified'.lower()),
                         rfc1123_date(mod_time))

    def test_index_html_with_304(self):
        path, _ref = self._extractFile('test_file.swf')
        mod_time = os.stat(path)[8]

        file = self._makeOne('test_file', 'test_file.swf')
        file = file.__of__(self.app)

        self.REQUEST.environ['IF_MODIFIED_SINCE'] = '%s;' % \
            rfc1123_date(mod_time + 3600)

        data = file.index_html(self.REQUEST, self.RESPONSE)

        self.assertEqual(data, '')
        # test that we don't supply a content-length
        self.assertEqual(self.RESPONSE.getHeader('Content-Length'.lower()),
                         None)
        self.assertEqual(self.RESPONSE.getStatus(), 304)

    def test_index_html_without_304(self):
        path, _ref = self._extractFile('test_file.swf')
        mod_time = os.stat(path)[8]

        file = self._makeOne('test_file', 'test_file.swf')
        file = file.__of__(self.app)

        self.REQUEST.environ['IF_MODIFIED_SINCE'] = '%s;' % \
            rfc1123_date(mod_time - 3600)

        data = file.index_html(self.REQUEST, self.RESPONSE)

        self.assertTrue(data, '')
        self.assertEqual(self.RESPONSE.getStatus(), 200)

    def test_index_html_with_304_from_cpm(self):
        cpm = DummyCachingManagerWithPolicy()
        getSiteManager().registerUtility(cpm, ICachingPolicyManager)
        path, _ref = self._extractFile('test_file.swf')
        file = self._makeOne('test_file', 'test_file.swf')
        file = file.__of__(self.app)

        mod_time = os.stat(path).st_mtime

        self.REQUEST.environ['IF_MODIFIED_SINCE'] = '%s;' % \
            rfc1123_date(mod_time)

        self.REQUEST.environ['IF_NONE_MATCH'] = '%s;' % FAKE_ETAG

        data = file.index_html(self.REQUEST, self.RESPONSE)
        self.assertEqual(len(data), 0)
        self.assertEqual(self.RESPONSE.getStatus(), 304)
        self.assertNotEqual(self.RESPONSE.getHeader('x-cache-headers-set-by'),
                            None)

    def test_index_html_200_with_cpm(self):
        # should behave the same as without cpm installed
        cpm = DummyCachingManagerWithPolicy()
        getSiteManager().registerUtility(cpm, ICachingPolicyManager)
        path, ref = self._extractFile('test_file.swf')
        file = self._makeOne('test_file', 'test_file.swf')
        file = file.__of__(self.app)

        mod_time = os.stat(path).st_mtime

        data = file.index_html(self.REQUEST, self.RESPONSE)

        self.assertEqual(len(data), len(ref))
        self.assertEqual(data, ref)
        # ICK!  'HTTPResponse.getHeader' doesn't case-flatten the key!
        self.assertEqual(self.RESPONSE.getHeader('Content-Length'.lower()),
                         str(len(ref)))
        self.assertEqual(self.RESPONSE.getHeader('Content-Type'.lower()),
                         'application/octet-stream')
        self.assertEqual(self.RESPONSE.getHeader('Last-Modified'.lower()),
                         rfc1123_date(mod_time))

    def test_caching(self):
        cpm = DummyCachingManager()
        getSiteManager().registerUtility(cpm, ICachingPolicyManager)
        original_len = len(self.RESPONSE.headers)
        obj = self._makeOne('test_file', 'test_file.swf')
        obj = obj.__of__(self.app)
        obj.index_html(self.REQUEST, self.RESPONSE)
        headers = self.RESPONSE.headers
        self.assertTrue(len(headers) >= original_len + 3)
        self.assertTrue('foo' in headers)
        self.assertTrue('bar' in headers)
        self.assertEqual(headers['test_path'], '/test_file')

    def test_forced_content_type(self):
        path, ref = self._extractFile('test_file_two.swf')
        mod_time = os.stat(path)[8]

        file = self._makeOne('test_file', 'test_file_two.swf')
        file = file.__of__(self.app)

        data = file.index_html(self.REQUEST, self.RESPONSE)

        self.assertEqual(len(data), len(ref))
        self.assertEqual(data, ref)
        # ICK!  'HTTPResponse.getHeader' doesn't case-flatten the key!
        self.assertEqual(self.RESPONSE.getHeader('Content-Length'.lower()),
                         str(len(ref)))
        self.assertEqual(self.RESPONSE.getHeader('Content-Type'.lower()),
                         'application/x-shockwave-flash')
        self.assertEqual(self.RESPONSE.getHeader('Last-Modified'.lower()),
                         rfc1123_date(mod_time))

    def test_utf8charset_detection(self):
        import mimetypes

        file_name = 'testUtf8.js'
        mtype, _ignore_enc = mimetypes.guess_type(file_name)
        file = self._makeOne(file_name, file_name)
        file = file.__of__(self.app)
        file.index_html(self.REQUEST, self.RESPONSE)
        self.assertEqual(self.RESPONSE.getHeader('content-type'),
                         '%s; charset=utf-8' % mtype)

    def test_unnecessary_invalidation_avoidance(self):
        # See https://bugs.launchpad.net/zope-cmf/+bug/325246
        invalidated = []

        def fake_invalidate(*args, **kw):
            invalidated.append(True)

        file = self._makeOne('test_file', 'test_file.swf')
        file.ZCacheable_invalidate = fake_invalidate

        # First pass: The internal file modification representation
        # equals the filesystem modification time.
        del invalidated[:]
        file._readFile(True)
        self.assertFalse(invalidated)

        del invalidated[:]
        file._parsed = False
        file._updateFromFS()
        self.assertFalse(invalidated)

        # Second pass: Forcing a different internal file modification
        # time onto the instance. Now the file will be invalidated.
        del invalidated[:]
        file._file_mod_time = 0
        file._readFile(True)
        self.assertTrue(invalidated)

        del invalidated[:]
        file._file_mod_time = 0
        file._parsed = False
        file._updateFromFS()
        self.assertTrue(invalidated)


def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(FSFileTests),
        ))



##############################################################################
#
# Copyright (c) 2002 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Implement a shared base for tools which provide actions.
"""

from warnings import warn

import six

from AccessControl.class_init import InitializeClass
from AccessControl.SecurityInfo import ClassSecurityInfo
from App.special_dtml import DTMLFile
from zope.interface import implementer

from .ActionInformation import ActionInfo
from .ActionInformation import ActionInformation
from .ActionInformation import getOAI
from .exceptions import AccessControl_Unauthorized
from .Expression import getExprContext
from .interfaces import IActionProvider
from .permissions import ManagePortal
from .utils import _dtmldir


@implementer(IActionProvider)
class ActionProviderBase:

    """ Provide ActionTabs and management methods for ActionProviders
    """

    security = ClassSecurityInfo()
    _actions = ()
    _actions_form = DTMLFile('editToolsActions', _dtmldir)

    manage_options = ({'label': 'Actions', 'action': 'manage_editActionsForm',
                       'help': ('CMFCore', 'Actions.stx')},)

    #
    #   ActionProvider interface
    #
    @security.private
    def listActions(self, info=None, object=None):
        """ List all the actions defined by a provider.
        """
        oldstyle_actions = self._actions or ()
        if oldstyle_actions:
            warn('Old-style actions are deprecated and will be removed in CMF '
                 '2.4. Use Action and Action Category objects instead.',
                 DeprecationWarning, stacklevel=2)

        return oldstyle_actions

    @security.private
    def getActionObject(self, action):
        """Return the actions object or None if action doesn't exist.
        """
        # separate cataegory and id from action
        sep = action.rfind('/')
        if sep == -1:
            raise ValueError('Actions must have the format <category>/<id>.')
        category, id = action[:sep], action[sep+1:]

        # search for action and return first one found
        for ai in self.listActions():
            try:
                if id == ai.getId() and category == ai.getCategory():
                    return ai
            except AttributeError:
                continue

        # no action found
        return None

    @security.public
    def listActionInfos(self, action_chain=None, object=None,
                        check_visibility=1, check_permissions=1,
                        check_condition=1, max=-1):
        # List ActionInfo objects.
        # (method is without docstring to disable publishing)
        #
        ec = self._getExprContext(object)
        actions = self.listActions(object=object)
        actions = [ActionInfo(action, ec) for action in actions]

        if action_chain:
            filtered_actions = []
            if isinstance(action_chain, six.string_types):
                action_chain = (action_chain,)
            for action_ident in action_chain:
                sep = action_ident.rfind('/')
                category, id = action_ident[:sep], action_ident[sep+1:]
                for ai in actions:
                    if id == ai['id'] and category == ai['category']:
                        filtered_actions.append(ai)
            actions = filtered_actions

        action_infos = []
        for ai in actions:
            if check_visibility and not ai['visible']:
                continue
            if check_permissions and not ai['allowed']:
                continue
            if check_condition and not ai['available']:
                continue
            action_infos.append(ai)
            if max + 1 and len(action_infos) >= max:
                break
        return action_infos

    @security.public
    def getActionInfo(self, action_chain, object=None, check_visibility=0,
                      check_condition=0):
        """ Get an ActionInfo object specified by a chain of actions.
        """
        action_infos = self.listActionInfos(action_chain, object,
                                            check_visibility=check_visibility,
                                            check_permissions=False,
                                            check_condition=check_condition)
        if not action_infos:
            if object is None:
                provider = self
            else:
                provider = object
            msg = 'Action "%s" not available for %s' % (
                        action_chain, '/'.join(provider.getPhysicalPath()))
            raise ValueError(msg)
        for ai in action_infos:
            if ai['allowed']:
                return ai
        raise AccessControl_Unauthorized('You are not allowed to access any '
                                         'of the specified Actions.')

    #
    #   ZMI methods
    #
    @security.protected(ManagePortal)
    def manage_editActionsForm(self, REQUEST, manage_tabs_message=None):

        """ Show the 'Actions' management tab.
        """
        actions = []
        for action in self.listActions():
            # The Actions tab currently only shows old-style actions,
            # so we need to weed out everything else.
            if getattr(action, 'getMapping', None) is not None:
                actions.append(action.getMapping())

        # possible_permissions is in AccessControl.Role.RoleManager.
        pp = self.possible_permissions()
        return self._actions_form(self, REQUEST, actions=actions,
                                  possible_permissions=pp,
                                  management_view='Actions',
                                  manage_tabs_message=manage_tabs_message)

    @security.protected(ManagePortal)
    def addAction(self, id, name, action, condition, permission, category,
                  visible=1, icon_expr='', link_target='', REQUEST=None):
        """ Add an action to our list.
        """
        if not name:
            raise ValueError('A name is required.')

        action = action and str(action) or ''
        condition = condition and str(condition) or ''

        if not isinstance(permission, tuple):
            permission = (str(permission),)

        new_actions = self._cloneActions()

        new_action = ActionInformation(id=str(id), title=str(name),
                                       category=str(category),
                                       condition=condition,
                                       permissions=permission,
                                       visible=bool(visible),
                                       action=action,
                                       icon_expr=icon_expr,
                                       link_target=link_target)

        new_actions.append(new_action)
        self._actions = tuple(new_actions)

        if REQUEST is not None:
            return self.manage_editActionsForm(
                REQUEST, manage_tabs_message='Added.')

    @security.protected(ManagePortal)
    def changeActions(self, properties=None, REQUEST=None):

        """ Update our list of actions.
        """
        if properties is None:
            properties = REQUEST

        actions = []

        for index in range(len(self._actions)):
            actions.append(self._extractAction(properties, index))

        self._actions = tuple(actions)

        if REQUEST is not None:
            msg = 'Actions changed.'
            return self.manage_editActionsForm(REQUEST,
                                               manage_tabs_message=msg)

    @security.protected(ManagePortal)
    def deleteActions(self, selections=(), REQUEST=None):

        """ Delete actions indicated by indexes in 'selections'.
        """
        sels = list(map(int, selections))  # Convert to a list of integers.

        old_actions = self._cloneActions()
        new_actions = []

        for index in range(len(old_actions)):
            if index not in sels:
                new_actions.append(old_actions[index])

        self._actions = tuple(new_actions)

        if REQUEST is not None:
            msg = 'Deleted %d action(s).' % len(sels)
            return self.manage_editActionsForm(REQUEST,
                                               manage_tabs_message=msg)

    @security.protected(ManagePortal)
    def moveUpActions(self, selections=(), REQUEST=None):

        """ Move the specified actions up one slot in our list.
        """
        sels = sorted(map(int, selections))  # Convert to list of ints

        new_actions = self._cloneActions()

        for idx in sels:
            idx2 = idx - 1
            if idx2 < 0:
                # Wrap to the bottom.
                idx2 = len(new_actions) - 1
            # Swap.
            a = new_actions[idx2]
            new_actions[idx2] = new_actions[idx]
            new_actions[idx] = a

        self._actions = tuple(new_actions)

        if REQUEST is not None:
            msg = 'Moved up %d action(s).' % len(sels)
            return self.manage_editActionsForm(REQUEST,
                                               manage_tabs_message=msg)

    @security.protected(ManagePortal)
    def moveDownActions(self, selections=(), REQUEST=None):

        """ Move the specified actions down one slot in our list.
        """
        sels = sorted(list(map(int, selections)))  # Convert to list of ints
        sels.reverse()

        new_actions = self._cloneActions()

        for idx in sels:
            idx2 = idx + 1
            if idx2 >= len(new_actions):
                # Wrap to the top.
                idx2 = 0
            # Swap.
            a = new_actions[idx2]
            new_actions[idx2] = new_actions[idx]
            new_actions[idx] = a

        self._actions = tuple(new_actions)

        if REQUEST is not None:
            msg = 'Moved down %d action(s).' % len(sels)
            return self.manage_editActionsForm(REQUEST,
                                               manage_tabs_message=msg)

    #
    #   Helper methods
    #
    @security.private
    def _cloneActions(self):

        """ Return a list of actions, cloned from our current list.
        """
        return [x.clone() for x in list(self._actions)]

    @security.private
    def _extractAction(self, properties, index):

        """ Extract an ActionInformation from the funky form properties.
        """
        id = str(properties.get('id_%d' % index, ''))
        title = str(properties.get('name_%d' % index, ''))
        action = str(properties.get('action_%d' % index, ''))
        icon_expr = str(properties.get('icon_expr_%d' % index, ''))
        condition = str(properties.get('condition_%d' % index, ''))
        category = str(properties.get('category_%d' % index, ''))
        visible = bool(properties.get('visible_%d' % index, False))
        permissions = properties.get('permission_%d' % index, ())
        link_target = str(properties.get('link_target_%d' % index, ''))

        if not title:
            raise ValueError('A title is required.')

        if category == '':
            category = 'object'

        if isinstance(permissions, six.string_types):
            permissions = (permissions,)

        return ActionInformation(id=id, title=title, action=action,
                                 condition=condition,
                                 permissions=permissions,
                                 category=category,
                                 visible=visible,
                                 icon_expr=icon_expr,
                                 link_target=link_target)

    def _getOAI(self, object):
        return getOAI(self, object)

    def _getExprContext(self, object):
        return getExprContext(self, object)


InitializeClass(ActionProviderBase)



ModifyPortalContent = 'Modify portal content'
View = 'View'

FTIDATA_ACTIONS = (
      {'id': 'Action Tests',
       'meta_type': 'Dummy',
       'aliases': {},
       'actions': ({'id': 'view',
                    'title': 'View',
                    'action': 'string:',
                    'permissions': ('View',),
                    'category': 'object',
                    'visible': 1},
                   {'id': 'edit',
                    'title': 'Edit',
                    'action': 'string:${object_url}/foo_edit',
                    'permissions': ('Modify',),
                    'category': 'object',
                    'visible': 1},
                   {'id': 'objectproperties',
                    'title': 'Object Properties',
                    'action': 'string:foo_properties',
                    'permissions': ('Modify',),
                    'category': 'object',
                    'visible': 1},
                   {'id': 'slot',
                    'title': 'Slot',
                    'action': 'string:foo_slot',
                    'category': 'object',
                    'visible': 0})},)

FTIDATA_DUMMY = (
      {'id': 'Dummy Content',
       'title': 'Dummy Content Title',
       'meta_type': 'Dummy',
       'product': 'FooProduct',
       'factory': 'addFoo',
       'aliases': {},
       'actions': ({'id': 'view',
                    'title': 'View',
                    'action': 'string:view',
                    'permissions': ('View',)},
                   {'id': 'view2',
                    'title': 'View2',
                    'action': 'string:view2',
                    'permissions': ('View',)},
                   {'id': 'edit',
                    'title': 'Edit',
                    'action': 'string:edit',
                    'permissions': ('forbidden permission',)})},)

FTIDATA_CMF15 = (
      {'id': 'Dummy Content 15',
       'meta_type': 'Dummy',
       'description': 'Dummy Content.',
       'icon': 'dummy_icon.gif',
       'product': 'FooProduct',
       'factory': 'addFoo',
       'immediate_view': 'metadata.html',
       'aliases': {'(Default)': 'dummy_view',
                   'view': 'dummy_view',
                   'view.html': 'dummy_view',
                   'edit.html': 'dummy_edit_form',
                   'metadata.html': 'metadata_edit_form',
                   'gethtml': 'source_html'},
       'actions': ({'id': 'view',
                    'title': 'View',
                    'action': 'string:${object_url}/view.html',
                    'permissions': (View,)},
                   {'id': 'edit',
                    'title': 'Edit',
                    'action': 'string:${object_url}/edit.html',
                    'permissions': (ModifyPortalContent,)},
                   {'id': 'metadata',
                    'title': 'Metadata',
                    'action': 'string:${object_url}/metadata.html',
                    'permissions': (ModifyPortalContent,)})},)

FTIDATA_CMF = (
      {'id': 'Dummy Content 15',
       'meta_type': 'Dummy',
       'description': 'Dummy Content.',
       'icon_expr': 'string:${portal_url}/dummy_icon.gif',
       'product': 'FooProduct',
       'factory': 'addFoo',
       'immediate_view': 'properties',
       'aliases': {'(Default)': 'dummy_view',
                   'view': 'dummy_view',
                   'edit': 'dummy_edit_form',
                   'properties': 'metadata_edit_form',
                   'gethtml': 'source_html'},
       'actions': ({'id': 'view',
                    'title': 'View',
                    'action': 'string:${object_url}',
                    'permissions': (View,)},
                   {'id': 'edit',
                    'title': 'Edit',
                    'action': 'string:${object_url}/edit',
                    'permissions': (ModifyPortalContent,)},
                   {'id': 'metadata',
                    'title': 'Metadata',
                    'action': 'string:${object_url}/properties',
                    'permissions': (ModifyPortalContent,)})},)

STI_SCRIPT = """\
## Script (Python) "addBaz"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=folder, id
##title=
##
product = folder.manage_addProduct['FooProduct']
product.addFoo(id)
item = getattr(folder, id)
return item
"""



##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Customizable image objects that come from the filesystem.
"""

import codecs
import os
from warnings import warn

import six

from AccessControl.class_init import InitializeClass
from AccessControl.SecurityInfo import ClassSecurityInfo
from App.special_dtml import DTMLFile
from OFS.Image import File
from zope.contenttype import guess_content_type
from ZPublisher.HTTPRequest import default_encoding

from .DirectoryView import registerFileExtension
from .DirectoryView import registerMetaType
from .FSObject import FSObject
from .permissions import FTPAccess
from .permissions import View
from .permissions import ViewManagementScreens
from .utils import _checkConditionalGET
from .utils import _dtmldir
from .utils import _FSCacheHeaders
from .utils import _setCacheHeaders
from .utils import _ViewEmulator


class FSFile(FSObject):

    """FSFiles act like images but are not directly
    modifiable from the management interface."""
    # Note that OFS.Image.File is not a base class because it is mutable.

    meta_type = 'Filesystem File'
    zmi_icon = 'far fa-file-archive'
    content_type = 'unknown/unknown'

    manage_options = ({'label': 'Customize', 'action': 'manage_main'},)

    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    security.declareProtected(ViewManagementScreens, 'manage_main')
    manage_main = DTMLFile('custfile', _dtmldir)

    def __init__(self, id, filepath, fullname=None, properties=None):
        id = fullname or id  # Use the whole filename.
        FSObject.__init__(self, id, filepath, fullname, properties)

    def _createZODBClone(self):
        return File(self.getId(), '', self._readFile(1))

    def _get_content_type(self, file, body, id, content_type=None):
        # Consult self.content_type first, this is either
        # the default (unknown/unknown) or it got a value from a
        # .metadata file
        default_type = 'unknown/unknown'
        if getattr(self, 'content_type', default_type) != default_type:
            return self.content_type

        # Next, look at file headers
        headers = getattr(file, 'headers', None)
        if headers and 'content-type' in headers:
            content_type = headers['content-type']
        else:
            # Last resort: Use the (imperfect) content type guessing
            # mechanism from OFS.Image, which ultimately uses the
            # Python mimetypes module.
            if not isinstance(body, (six.string_types, six.binary_type)):
                body = body.data
            content_type, enc = guess_content_type(
                getattr(file, 'filename', id), body, content_type)
            if (enc is None
                and (content_type.startswith('text/') or
                     content_type.startswith('application/'))
                    and body.startswith(codecs.BOM_UTF8)):
                content_type += '; charset=utf-8'

        return content_type

    def _readFile(self, reparse):
        """Read the data from the filesystem.
        """
        file = open(self._filepath, 'rb')
        try:
            data = file.read()
        finally:
            file.close()

        if reparse or self.content_type == 'unknown/unknown':
            try:
                mtime = os.stat(self._filepath).st_mtime
            except Exception:
                mtime = 0.0
            if mtime != self._file_mod_time or mtime == 0.0:
                self.ZCacheable_invalidate()
                self._file_mod_time = mtime
            self.content_type = self._get_content_type(file, data, self.id)
        return data

    # The following is mainly taken from OFS/File.py

    def __str__(self):
        self._updateFromFS()
        if six.PY2:
            return str(self._readFile(0))

        data = self._readFile(0)
        ct = self.content_type
        encoding = None

        if 'charset=' in ct:
            encoding = ct[ct.find('charset=')+8:]
        elif getattr(self, 'encoding', None):
            encoding = self.encoding
        elif ct.startswith('text/'):
            encoding = default_encoding

        if encoding:
            return str(data, encoding=encoding)

        warn('Calling str() on non-text data is deprecated, use bytes()',
             DeprecationWarning, stacklevel=2)
        return str(data, encoding=default_encoding)

    def __bytes__(self):
        self._updateFromFS()
        return bytes(self._readFile(0))

    def modified(self):
        return self.getModTime()

    @security.protected(View)
    def index_html(self, REQUEST, RESPONSE):
        """
        The default view of the contents of a File or Image.

        Returns the contents of the file or image.  Also, sets the
        Content-Type HTTP header to the objects content type.
        """
        self._updateFromFS()
        view = _ViewEmulator().__of__(self)

        # There are 2 Cache Managers which can be in play....
        # need to decide which to use to determine where the cache headers
        # are decided on.
        if self.ZCacheable_getManager() is not None:
            self.ZCacheable_set(None)
        else:
            _setCacheHeaders(view, extra_context={})

        # If we have a conditional get, set status 304 and return
        # no content
        if _checkConditionalGET(view, extra_context={}):
            return ''

        RESPONSE.setHeader('Content-Type', self.content_type)

        # old-style If-Modified-Since header handling.
        if self._setOldCacheHeaders():
            # Make sure the CachingPolicyManager gets a go as well
            _setCacheHeaders(view, extra_context={})
            return ''

        data = self._readFile(0)
        data_len = len(data)
        RESPONSE.setHeader('Content-Length', data_len)
        return data

    def _setOldCacheHeaders(self):
        # return False to disable this simple caching behaviour
        return _FSCacheHeaders(self)

    @security.protected(View)
    def getContentType(self):
        """Get the content type of a file or image.

        Returns the content type (MIME type) of a file or image.
        """
        self._updateFromFS()
        return self.content_type

    security.declareProtected(FTPAccess, 'manage_FTPget')
    manage_FTPget = index_html


InitializeClass(FSFile)

registerFileExtension('doc', FSFile)
registerFileExtension('txt', FSFile)
registerFileExtension('pdf', FSFile)
registerFileExtension('swf', FSFile)
registerFileExtension('jar', FSFile)
registerFileExtension('cab', FSFile)
registerFileExtension('ico', FSFile)
registerFileExtension('js', FSFile)
registerFileExtension('css', FSFile)
registerFileExtension('map', FSFile)
registerFileExtension('svg', FSFile)
registerFileExtension('ttf', FSFile)
registerFileExtension('eot', FSFile)
registerFileExtension('woff', FSFile)
registerFileExtension('woff2', FSFile)
registerMetaType('File', FSFile)



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" Unit test mixin classes and layers.
"""

from OFS.SimpleItem import SimpleItem
from Testing.ZopeTestCase.layer import ZopeLite
from Zope2.App import zcml
from zope.component import adapts
from zope.component.hooks import setHooks
from zope.i18n.interfaces import IUserPreferredLanguages
from zope.interface import implementer
from zope.interface.verify import verifyClass
from zope.publisher.interfaces.http import IHTTPRequest
from zope.testing.cleanup import cleanUp

from Products.GenericSetup.utils import BodyAdapterBase

from .interfaces import IWorkflowDefinition
from .utils import HAS_ZSERVER


class ConformsToFolder:

    def test_conforms_to_IWritelock(self):
        try:
            from OFS.interfaces import IWriteLock
        except ImportError:
            from webdav.interfaces import IWriteLock
        verifyClass(IWriteLock, self._getTargetClass())

    def test_conforms_to_IDynamicType(self):
        from .interfaces import IDynamicType
        verifyClass(IDynamicType, self._getTargetClass())

    def test_conforms_to_IFolderish(self):
        from .interfaces import IFolderish
        verifyClass(IFolderish, self._getTargetClass())

    def test_conforms_to_IMutableDublinCore(self):
        from .interfaces import IMutableMinimalDublinCore
        verifyClass(IMutableMinimalDublinCore, self._getTargetClass())

    def test_folder_extra_interfaces(self):
        # in the long run this interface will become deprecated
        from .interfaces import IOpaqueItemManager

        verifyClass(IOpaqueItemManager, self._getTargetClass())


class ConformsToContent:

    def test_content_interfaces(self):
        from Products.GenericSetup.interfaces import IDAVAware

        from .interfaces import ICatalogableDublinCore
        from .interfaces import IContentish
        from .interfaces import IDublinCore
        from .interfaces import IDynamicType
        from .interfaces import IMutableDublinCore

        verifyClass(ICatalogableDublinCore, self._getTargetClass())
        verifyClass(IContentish, self._getTargetClass())
        verifyClass(IDublinCore, self._getTargetClass())
        verifyClass(IDynamicType, self._getTargetClass())
        verifyClass(IMutableDublinCore, self._getTargetClass())
        if HAS_ZSERVER:
            verifyClass(IDAVAware, self._getTargetClass())

    def test_content_extra_interfaces(self):
        # in the long run these interfaces will become deprecated
        from .interfaces import ICatalogAware
        from .interfaces import IOpaqueItemManager
        from .interfaces import IWorkflowAware

        verifyClass(ICatalogAware, self._getTargetClass())
        verifyClass(IOpaqueItemManager, self._getTargetClass())
        verifyClass(IWorkflowAware, self._getTargetClass())


@implementer(IUserPreferredLanguages)
class BrowserLanguages(object):

    adapts(IHTTPRequest)

    def __init__(self, context):
        self.context = context

    def getPreferredLanguages(self):
        return ('test',)


class EventZCMLLayer(ZopeLite):

    @classmethod
    def testSetUp(cls):
        import OFS

        import Products

        zcml.load_config('meta.zcml', Products.Five)
        zcml.load_config('event.zcml', OFS)
        zcml.load_config('event.zcml', Products.CMFCore)
        setHooks()

    @classmethod
    def testTearDown(cls):
        cleanUp()


class TraversingZCMLLayer(ZopeLite):

    @classmethod
    def testSetUp(cls):
        import Products.Five
        import zope.traversing

        zcml.load_config('meta.zcml', Products.Five)
        zcml.load_config('configure.zcml', zope.traversing)
        setHooks()

    @classmethod
    def testTearDown(cls):
        cleanUp()


class TraversingEventZCMLLayer(ZopeLite):

    @classmethod
    def testSetUp(cls):
        import OFS
        import Products.Five
        import zope.traversing

        zcml.load_config('meta.zcml', Products.Five)
        zcml.load_config('configure.zcml', zope.traversing)
        zcml.load_config('event.zcml', OFS)
        zcml.load_config('event.zcml', Products.CMFCore)
        zcml.load_config('tool.zcml', Products.CMFCore)
        setHooks()

    @classmethod
    def testTearDown(cls):
        cleanUp()


class FunctionalZCMLLayer(ZopeLite):

    @classmethod
    def setUp(cls):
        import Products

        zcml.load_config('testing.zcml', Products.CMFCore)
        setHooks()

    @classmethod
    def tearDown(cls):
        cleanUp()


_DUMMY_ZCML = """\
<configure
    xmlns="http://namespaces.zope.org/zope"
    xmlns:five="http://namespaces.zope.org/five"
    i18n_domain="dummy">
  <permission id="dummy.add" title="Add Dummy Workflow"/>
  <five:registerClass
      class="Products.CMFCore.testing.DummyWorkflow"
      meta_type="Dummy Workflow"
      permission="dummy.add"
      addview="addDummyWorkflow.html"
      global="false"
      />
  <adapter
      factory="Products.CMFCore.testing.DummyWorkflowBodyAdapter"
      provides="Products.GenericSetup.interfaces.IBody"
      for="Products.CMFCore.interfaces.IWorkflowDefinition
           Products.GenericSetup.interfaces.ISetupEnviron"
      />
</configure>
"""


@implementer(IWorkflowDefinition)
class DummyWorkflow(SimpleItem):

    meta_type = 'Dummy Workflow'

    def __init__(self, id):
        self._id = id

    def getId(self):
        return self._id


class DummyWorkflowBodyAdapter(BodyAdapterBase):

    body = property(BodyAdapterBase._exportBody, BodyAdapterBase._importBody)


class ExportImportZCMLLayer(ZopeLite):

    @classmethod
    def testSetUp(cls):
        import AccessControl
        import Products.Five
        import Zope2.App

        import Products.CMFCore
        import Products.CMFCore.exportimport
        import Products.GenericSetup

        zcml.load_config('meta.zcml', Zope2.App)
        zcml.load_config('meta.zcml', Products.Five)
        zcml.load_config('permissions.zcml', AccessControl)
        zcml.load_config('permissions.zcml', Products.Five)
        zcml.load_config('meta.zcml', Products.GenericSetup)
        zcml.load_config('configure.zcml', Products.GenericSetup)
        zcml.load_config('permissions.zcml', Products.CMFCore)
        zcml.load_config('tool.zcml', Products.CMFCore)
        zcml.load_config('configure.zcml', Products.CMFCore.exportimport)
        zcml.load_string(_DUMMY_ZCML)
        setHooks()

    @classmethod
    def testTearDown(cls):
        cleanUp()
